/*
 * Copyright (C) 2011 Kouichi Machida (k01-dev@live.jp)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package jp.white_wing.android.appwidget.electricpowerconsumption;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.Calendar;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.util.EntityUtils;

import android.app.AlarmManager;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.appwidget.AppWidgetManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Typeface;
import android.graphics.drawable.GradientDrawable;
import android.os.IBinder;
import android.preference.PreferenceManager;
import android.text.SpannableStringBuilder;
import android.text.Spanned;
import android.text.style.AbsoluteSizeSpan;
import android.text.style.ForegroundColorSpan;
import android.text.style.StyleSpan;
import android.util.Log;
import android.widget.RemoteViews;

/**
 * d͏ʃBWFbg̎又sT[rX񋟂܂B
 * 
 * @author Kouichi Machida
 */
public class ElectricPowerConsumptionAppWidgetService extends Service implements SharedPreferences.OnSharedPreferenceChangeListener {

	/**
	 * 
	 */
	public static final String ACTION_APPWIDGET_CLICK = "jp.white_wing.android.appwidget.electricpowerconsumption.ElectricPowerConsumptionAppWidgetService.ACTION_APPWIDGET_CLICK";

	/**
	 * BWFbg̃f[^ƕ\XV܂B
	 */
	public static final String ACTION_APPWIDGET_INVALIDATE = "jp.white_wing.android.appwidget.electricpowerconsumption.ElectricPowerConsumptionAppWidgetService.ACTION_APPWIDGET_INVALIDATE";

	/**
	 * BWFbg̕\XV܂B
	 */
	public static final String ACTION_APPWIDGET_REFRESH = "jp.white_wing.android.appwidget.electricpowerconsumption.ElectricPowerConsumptionAppWidgetService.ACTION_APPWIDGET_REFRESH";

	/**
	 * 
	 */
	public static final String ACTION_NOTIFICATION_CLEAR = "jp.white_wing.android.appwidget.electricpowerconsumption.ElectricPowerConsumptionAppWidgetService.ACTION_NOTIFICATION_CLEAR";

	/**
	 * 
	 */
	public static final String ACTION_NOTIFICATION_CLICK = "jp.white_wing.android.appwidget.electricpowerconsumption.ElectricPowerConsumptionAppWidgetService.ACTION_NOTIFICATION_CLICK";

	/**
	 * 
	 */
	public static final String ACTION_PREFERENCE_INTERVAL_UPDATE = "jp.white_wing.android.appwidget.electricpowerconsumption.ElectricPowerConsumptionAppWidgetService.ACTION_PREFERENCE_INTERVAL_UPDATE";

	/**
	 * 
	 */
	public static final String ACTION_UPDATE = "jp.white_wing.android.appwidget.electricpowerconsumption.ElectricPowerConsumptionAppWidgetService.ACTION_UPDATE";

	/**
	 * 
	 */
	private static final String CSV_SEPARATOR = ",";

	/**
	 * 
	 */
	private static final String CSV_SEPARATOR_SPACE = " ";

	/**
	 * 
	 */
	private static final String CSV_STARTWITH_DATE = "DATE";

	/**
	 * 
	 */
	private static final String CSV_STARTWITH_EXPECTED_POWER = "\ződ(kW)";

	/**
	 * 
	 */
	private static final String CSV_STARTWITH_PEAK_POWER = "s[N(kW)";

	/**
	 * f[^t@C̃GR[fBO\܂B
	 */
	private static final String DATA_ENCODING = "Windows-31J";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_BACKGROUND_ENABLED = "electric_powerconsumption_appwidget_background_enabled";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_FONT_BOLD = "electric_powerconsumption_appwidget_consumption_rate_font_bold_enabled";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_FONT_COLOR = "electric_powerconsumption_appwidget_consumption_rate_font_color";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_FULL_FONT_COLOR = "electric_powerconsumption_appwidget_consumption_rate_full_font_color";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_GRADATION_FONT_COLOR_ENABLED = "electric_powerconsumption_appwidget_consumption_rate_gradation_font_color_enabled";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_HALF_FONT_COLOR = "electric_powerconsumption_appwidget_consumption_rate_half_font_color";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_ZERO_FONT_COLOR = "electric_powerconsumption_appwidget_consumption_rate_zero_font_color";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_NETWORK_INTERVAL = "electric_powerconsumption_appwidget_network_interval";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_NETWORK_URL = "electric_powerconsumption_appwidget_network_url";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_NOTIFICATION_TIMEOUT = "electric_powerconsumption_appwidget_notification_timeout";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_SUMMARY_FONT_BOLD = "electric_powerconsumption_appwidget_summary_font_bold";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_SUMMARY_FONT_COLOR = "electric_powerconsumption_appwidget_summary_font_color";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_TAP_FUNCTION = "electric_powerconsumption_appwidget_tap_function";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_TAP_FUNCTION_CONFIG = "config";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_TAP_FUNCTION_INTENT = "intent";

	/**
	 * 
	 */
	public static final String ELECTRIC_POWERCONSUMPTION_APPWIDGET_TAP_FUNCTION_REFRESH = "refresh";

	/**
	 * 
	 */
	public static final String EMPTY_STRING = "";

	/**
	 * Õ^O\܂B
	 */
	private static final String TAG = "ElectricPowerConsumptionAppWidgetService";

	/**
	 * wiC[Wێ Bitmap IuWFNgB
	 */
	private Bitmap backgroundBitmap = null;

	/**
	 * XVɔOێ܂B
	 */
	private Exception exception = null;

	/**
	 * ExecutorService IuWFNgێ܂B
	 */
	private ExecutorService executorService = null;

	/**
	 * ǂݍݒǂێ܂B
	 */
	private boolean isLoading = false;

	/**
	 * \Ԃǂێ܂B
	 */
	private boolean isReady = false;

	/**
	 * ŏIXVێ܂B
	 */
	private long lastModificationTime = 0;

	/**
	 * s[N̋ʂێ܂B
	 */
	private float peak;

	/**
	 * ݂̗pʂێ܂B
	 */
	private float today;

	/**
	 * ȃC[Wێ Bitmap IuWFNgB
	 */
	private Bitmap transparentBitmp = null;

	/**
	 * XVێ܂B
	 */
	private String updateDate = null;

	/**
	 * XVێ܂B
	 */
	private String updateTime = null;

	/**
	 * ̗pʂێ܂B
	 */
	private float yesterday;

	/**
	 * Notification ̎N[Yݒ肵܂B
	 */
	private void autoCancelNotification() {
		// Resources, SharedPreferences IuWFNg擾܂B
		Resources resources = this.getResources();
		SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

		// ݒ擾܂B
		int electric_powerconsumption_appwidget_notification_timeout_millis = sharedPreferences.getInt(ELECTRIC_POWERCONSUMPTION_APPWIDGET_NOTIFICATION_TIMEOUT, resources.getInteger(R.integer.electric_powerconsumption_appwidget_notification_timeout_millis_default));

		// AlarmManager Ƀs[g^C}[ݒ肵܂B
		AlarmManager alarmManager = (AlarmManager) this.getSystemService(Context.ALARM_SERVICE);

		Calendar calendar = Calendar.getInstance();

		calendar.setTimeInMillis(System.currentTimeMillis());
		calendar.add(Calendar.MILLISECOND, electric_powerconsumption_appwidget_notification_timeout_millis);

		// Ώۂ Intent ܂B
		PendingIntent pendingIntent = PendingIntent.getService(this, 0, new Intent(ACTION_NOTIFICATION_CLEAR), PendingIntent.FLAG_CANCEL_CURRENT);

		alarmManager.cancel(pendingIntent);
		alarmManager.set(AlarmManager.RTC, calendar.getTimeInMillis(), pendingIntent);
	}

	/**
	 * \ĂʒmׂăLZ܂B
	 * 
	 * @param context
	 *            AvP[ṼReLXgB
	 */
	private void cancelAllNotification() {
		// LOG:
		Log.v(TAG, "void cancelAllNotification()");

		NotificationManager notificationManager = (NotificationManager) this.getSystemService(Context.NOTIFICATION_SERVICE);

		notificationManager.cancelAll();
	}

	/**
	 * wiC[Wێ Bitmap IuWFNg擾܂B
	 * 
	 * @return wiC[Wێ Bitmap IuWFNgB
	 */
	private Bitmap getBackgroundBitmap() {
		// wiC[W\zς݂mF܂B
		if (this.backgroundBitmap == null) {
			try {
				Resources resources = this.getResources();
				SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

				// BWFbg̕\TCY擾܂B
				int minWidth = resources.getDimensionPixelSize(R.dimen.electric_powerconsumption_appwidget_minwidth);
				int minHeight = resources.getDimensionPixelSize(R.dimen.electric_powerconsumption_appwidget_minheight);

				// ݒ肳ꂽF擾܂B
				int color = sharedPreferences.getInt("electric_powerconsumption_appwidget_background_color", resources.getColor(R.color.electric_powerconsumption_appwidget_background_color_default));

				// drawable 擾܂B
				GradientDrawable electric_powerconsumption_appwidget_background = (GradientDrawable) resources.getDrawable(R.drawable.electric_powerconsumption_appwidget_background);

				// ̈ƐFݒ肵܂B
				electric_powerconsumption_appwidget_background.setBounds(0, 0, minWidth, minHeight);
				electric_powerconsumption_appwidget_background.setColor(color);

				// Bitmap IuWFNg܂B
				Bitmap bitmap = Bitmap.createBitmap(minWidth, minHeight, Bitmap.Config.ARGB_4444);

				//  Canvas ܂B
				Canvas canvas = new Canvas(bitmap);

				// drawable `悵܂B
				electric_powerconsumption_appwidget_background.draw(canvas);

				this.backgroundBitmap = bitmap;
			} catch (Exception ex) {
				// LOG:
				Log.e(TAG, String.format("Exception message is %s.", ex.getMessage()));

				this.backgroundBitmap = null;
			}
		}

		return this.backgroundBitmap;
	}

	/**
	 * ȃC[Wێ Bitmap IuWFNg擾܂B
	 * 
	 * @return ȃC[Wێ Bitmap IuWFNgB
	 */
	private Bitmap getTransparentBitmap() {
		// ȃC[W\zς݂mF܂B
		if (this.transparentBitmp == null) {
			// Bitmap IuWFNg܂B
			Bitmap bitmap = Bitmap.createBitmap(0xf, 0xf, Bitmap.Config.ARGB_4444);

			//  Bitmap ܂B
			bitmap.eraseColor(Color.argb(0, 0xff, 0xff, 0xff));

			this.transparentBitmp = bitmap;
		}

		return this.transparentBitmp;
	}

	/**
	 * f[^̍XVsABWFbg̕\XV܂B
	 */
	public void invalidate() {
		// LOG:
		Log.v(TAG, "void invalidate()");

		this.update();

		// BWFbg̕\XV܂B
		this.refresh();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Service#onBind(android.content.Intent)
	 */
	@Override
	public IBinder onBind(Intent intent) {
		return null;
	}

	/**
	 * 
	 */
	private void onClick() {
		// LOG:
		Log.v(TAG, "onClick()");

		SharedPreferences preferences = PreferenceManager.getDefaultSharedPreferences(this);

		// ݒl擾܂B
		String electric_powerconsumption_appwidget_tap_function = preferences.getString(ELECTRIC_POWERCONSUMPTION_APPWIDGET_TAP_FUNCTION, this.getString(R.string.electric_powerconsumption_appwidget_tap_function_default));

		// LOG:
		Log.v(TAG, electric_powerconsumption_appwidget_tap_function);

		Resources resources = this.getResources();

		StringBuilder text = new StringBuilder();

		text.append(resources.getString(R.string.electric_powerconsumption_appwidget_consumption_rate_text_format, (this.today / this.peak) * 100f));
		text.append(resources.getString(R.string.electric_powerconsumption_appwidget_summary_peak_text_format, this.peak));
		text.append(resources.getString(R.string.electric_powerconsumption_appwidget_summary_today_text_format, this.today));
		text.append(resources.getString(R.string.electric_powerconsumption_appwidget_summary_yesterday_text_format, this.yesterday));
		text.append(resources.getString(R.string.electric_powerconsumption_appwidget_summary_datetime_text_format, this.updateDate, this.updateTime));

		Intent intent = new Intent();

		intent.setAction(Intent.ACTION_SEND);
		intent.setType("text/plain");
		intent.putExtra(Intent.EXTRA_TEXT, text.toString());
		intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

		this.startActivity(intent);

		/*
ELECTRIC_POWERCONSUMPTION_APPWIDGET_TAP_FUNCTION_INTENT
		if (ELECTRIC_POWERCONSUMPTION_APPWIDGET_TAP_FUNCTION_REFRESH.equals(electric_powerconsumption_appwidget_tap_function)) {
			this.invalidate();
		} else if (ELECTRIC_POWERCONSUMPTION_APPWIDGET_TAP_FUNCTION_CONFIG.equals(electric_powerconsumption_appwidget_tap_function)) {
			Intent intent = new Intent(this, ElectricPowerConsumptionAppWidgetPreferenceActivity.class);

			intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);

			this.startActivity(intent);
		}
		*/
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Service#onConfigurationChanged(android.content.res.Configuration)
	 */
	@Override
	public void onConfigurationChanged(Configuration newConfig) {
		// LOG:
		Log.v(TAG, "void onConfigurationChanged(Configuration)");

		super.onConfigurationChanged(newConfig);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Service#onCreate()
	 */
	@Override
	public void onCreate() {
		// LOG:
		Log.v(TAG, "void onCreate()");

		// ExecutorService IuWFNg܂B
		this.executorService = Executors.newCachedThreadPool();

		// AlarmManager ݒ肵܂B
		this.registerAlarmManager();

		// ݒύXm悤ݒ肵܂B
		PreferenceManager.getDefaultSharedPreferences(this).registerOnSharedPreferenceChangeListener(this);

		// NGXgɓ܂B
		this.startService(new Intent(ACTION_APPWIDGET_INVALIDATE, null, this, ElectricPowerConsumptionAppWidgetService.class));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Service#onDestroy()
	 */
	@Override
	public void onDestroy() {
		// LOG:
		Log.v(TAG, "void onDestroy()");

		// ݒύX̌m܂B
		PreferenceManager.getDefaultSharedPreferences(this).unregisterOnSharedPreferenceChangeListener(this);

		// AlarmManager ̐ݒ܂B
		this.unregisterAlarmManager();

		// ExecutorService J܂B
		this.executorService.shutdown();
		this.executorService = null;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Service#onLowMemory()
	 */
	@Override
	public void onLowMemory() {
		// LOG:
		Log.v(TAG, "void onLowMemory()");

		super.onLowMemory();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Service#onRebind(android.content.Intent)
	 */
	@Override
	public void onRebind(Intent intent) {
		// LOG:
		Log.v(TAG, "void onRebind(Intent)");

		super.onRebind(intent);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.content.SharedPreferences.OnSharedPreferenceChangeListener#onSharedPreferenceChanged(android.content.SharedPreferences, java.lang.String)
	 */
	@Override
	public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
		// LOG:
		Log.v(TAG, "void onSharedPreferenceChanged(SharedPreferences, String)");
		Log.v(TAG, String.format("key = %s", key));

		if (ELECTRIC_POWERCONSUMPTION_APPWIDGET_NETWORK_INTERVAL.equals(key)) {
			this.startService(new Intent(ACTION_PREFERENCE_INTERVAL_UPDATE, null, this, ElectricPowerConsumptionAppWidgetService.class));
		} else {
			this.startService(new Intent(ACTION_APPWIDGET_REFRESH, null, this, ElectricPowerConsumptionAppWidgetService.class));
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Service#onStartCommand(android.content.Intent, int, int)
	 */
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		// LOG:
		Log.v(TAG, "int onStartCommand(Intent, int, int)");
		Log.v(TAG, String.format("intent = %s, flags = %d, startId = %d", intent, flags, startId));

		final String action = intent.getAction();

		if (ACTION_APPWIDGET_CLICK.equals(action)) {
			// XbhŎs܂B
			this.executorService.execute(new Runnable() {

				/*
				 * (non-Javadoc)
				 * 
				 * @see java.lang.Runnable#run()
				 */
				@Override
				public void run() {
					ElectricPowerConsumptionAppWidgetService.this.onClick();
				}

			});

			return Service.START_STICKY;
		} else if (ACTION_APPWIDGET_REFRESH.equals(action)) {
			// XbhŎs܂B
			this.executorService.execute(new Runnable() {

				/*
				 * (non-Javadoc)
				 * 
				 * @see java.lang.Runnable#run()
				 */
				@Override
				public void run() {
					ElectricPowerConsumptionAppWidgetService.this.refresh();
				}

			});

			return Service.START_STICKY;
		} else if (ACTION_APPWIDGET_INVALIDATE.equals(action)) {
			// XbhŎs܂B
			this.executorService.execute(new Runnable() {

				/*
				 * (non-Javadoc)
				 * 
				 * @see java.lang.Runnable#run()
				 */
				@Override
				public void run() {
					ElectricPowerConsumptionAppWidgetService.this.invalidate();
				}

			});

			return Service.START_STICKY;
		} else if (ACTION_PREFERENCE_INTERVAL_UPDATE.equals(action)) {
			// XbhŎs܂B
			this.executorService.execute(new Runnable() {

				/*
				 * (non-Javadoc)
				 * 
				 * @see java.lang.Runnable#run()
				 */
				@Override
				public void run() {
					ElectricPowerConsumptionAppWidgetService.this.refreshAlarmManager();
				}

			});

			return Service.START_STICKY;
		} else if (ACTION_NOTIFICATION_CLEAR.equals(action)) {
			// XbhŎs܂B
			this.executorService.execute(new Runnable() {

				/*
				 * (non-Javadoc)
				 * 
				 * @see java.lang.Runnable#run()
				 */
				@Override
				public void run() {
					ElectricPowerConsumptionAppWidgetService.this.cancelAllNotification();
				}

			});

			return Service.START_STICKY;
		} else if (ACTION_NOTIFICATION_CLICK.equals(action)) {
			// ɉ܂B
			return Service.START_STICKY;
		} else if (ACTION_UPDATE.equals(action)) {
			// XbhŎs܂B
			this.executorService.execute(new Runnable() {

				/*
				 * (non-Javadoc)
				 * 
				 * @see java.lang.Runnable#run()
				 */
				@Override
				public void run() {
					ElectricPowerConsumptionAppWidgetService.this.update();
				}

			});

			return Service.START_STICKY;
		} else {
			return Service.START_STICKY;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see android.app.Service#onUnbind(android.content.Intent)
	 */
	@Override
	public boolean onUnbind(Intent intent) {
		// LOG:
		Log.v(TAG, "boolean onUnbind(Intent)");

		return super.onUnbind(intent);
	}

	/**
	 * BWFbg̕\XV܂B
	 */
	public void refresh() {
		// LOG:
		Log.v(TAG, "void refresh()");

		if ((this.exception == null) && !this.isLoading && this.isReady) {
			this.refreshResult();
		} else if (!this.isReady || this.isLoading) {
			this.refreshLoading();
		} else if (this.exception != null) {
			this.refreshError();
		}
	}

	/**
	 * AlarmManager ݒeɉčēxݒ肵܂B
	 */
	private void refreshAlarmManager() {
		// LOG:
		Log.v(TAG, "void refreshAlarmManager()");

		// ݒ܂B
		this.unregisterAlarmManager();

		// ēxݒsȂ܂B
		this.registerAlarmManager();
	}

	/**
	 * G[\܂B
	 */
	private void refreshError() {
		// LOG:
		Log.v(TAG, "void refreshError()");

		Resources resources = this.getResources();
		SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

		// ݒl擾܂B
		boolean electric_powerconsumption_appwidget_background_enabled = sharedPreferences.getBoolean(ELECTRIC_POWERCONSUMPTION_APPWIDGET_BACKGROUND_ENABLED, resources.getBoolean(R.bool.electric_powerconsumption_appwidget_background_enabled_default));
		boolean electric_powerconsumption_appwidget_summary_font_bold = sharedPreferences.getBoolean(ELECTRIC_POWERCONSUMPTION_APPWIDGET_SUMMARY_FONT_BOLD, resources.getBoolean(R.bool.electric_powerconsumption_appwidget_summary_font_bold_default));
		int electric_powerconsumption_appwidget_summary_font_color = sharedPreferences.getInt(ELECTRIC_POWERCONSUMPTION_APPWIDGET_SUMMARY_FONT_COLOR, resources.getColor(R.color.electric_powerconsumption_appwidget_summary_font_color_default));
		int electric_powerconsumption_appwidget_summary_font_size = resources.getDimensionPixelSize(R.dimen.electric_powerconsumption_appwidget_summary_font_size);

		// RemoteViews ܂B
		RemoteViews remoteViews = new RemoteViews(this.getPackageName(), R.layout.electric_powerconsumption_appwidget);

		// wi̐ݒsȂ܂B
		if (electric_powerconsumption_appwidget_background_enabled) {
			remoteViews.setImageViewBitmap(R.id.electric_powerconsumption_appwidget_background, this.getBackgroundBitmap());
		} else {
			remoteViews.setImageViewBitmap(R.id.electric_powerconsumption_appwidget_background, this.getTransparentBitmap());
		}

		// ڍ׏ SpannableStringBuilder ܂B
		SpannableStringBuilder electric_powerconsumption_appwidget_summary = new SpannableStringBuilder();

		// ڍ׏̕\\z܂B
		electric_powerconsumption_appwidget_summary.append(this.getString(R.string.electric_powerconsumption_appwidget_summary_loading_text));
		electric_powerconsumption_appwidget_summary.setSpan(new ForegroundColorSpan(electric_powerconsumption_appwidget_summary_font_color), 0, electric_powerconsumption_appwidget_summary.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
		electric_powerconsumption_appwidget_summary.setSpan(new AbsoluteSizeSpan(electric_powerconsumption_appwidget_summary_font_size, true), 0, electric_powerconsumption_appwidget_summary.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);

		// ɂ܂B
		if (electric_powerconsumption_appwidget_summary_font_bold) {
			electric_powerconsumption_appwidget_summary.setSpan(new StyleSpan(Typeface.BOLD), 0, electric_powerconsumption_appwidget_summary.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
		}

		// r[ɃeLXgƃCxgݒ肵܂B
		remoteViews.setTextViewText(R.id.electric_powerconsumption_appwidget_consumption_rate, EMPTY_STRING);
		remoteViews.setTextViewText(R.id.electric_powerconsumption_appwidget_summary, electric_powerconsumption_appwidget_summary);
		remoteViews.setTextViewText(R.id.electric_powerconsumption_appwidget_update_time, EMPTY_STRING);
		remoteViews.setOnClickPendingIntent(R.id.electric_powerconsumption_appwidget_consumption_rate, PendingIntent.getService(this, 0, new Intent(ACTION_APPWIDGET_CLICK), 0));
		remoteViews.setOnClickPendingIntent(R.id.electric_powerconsumption_appwidget_summary, PendingIntent.getService(this, 0, new Intent(ACTION_APPWIDGET_CLICK), 0));

		// AppWidgetManager ܂B
		AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(this);

		// BWFbgXV܂B
		appWidgetManager.updateAppWidget(new ComponentName(this, ElectricPowerConsumptionAppWidgetProvider.class), remoteViews);
	}

	/**
	 * 
	 */
	private void refreshLoading() {
		// LOG:
		Log.v(TAG, "void refreshLoading()");

		// Resources, SharedPreferences IuWFNg擾܂B
		Resources resources = this.getResources();
		SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

		// ݒ擾܂B
		boolean electric_powerconsumption_appwidget_background_enabled = sharedPreferences.getBoolean(ELECTRIC_POWERCONSUMPTION_APPWIDGET_BACKGROUND_ENABLED, resources.getBoolean(R.bool.electric_powerconsumption_appwidget_background_enabled_default));
		boolean electric_powerconsumption_appwidget_summary_font_bold = sharedPreferences.getBoolean(ELECTRIC_POWERCONSUMPTION_APPWIDGET_SUMMARY_FONT_BOLD, resources.getBoolean(R.bool.electric_powerconsumption_appwidget_summary_font_bold_default));
		int electric_powerconsumption_appwidget_summary_font_color = sharedPreferences.getInt(ELECTRIC_POWERCONSUMPTION_APPWIDGET_SUMMARY_FONT_COLOR, resources.getColor(R.color.electric_powerconsumption_appwidget_summary_font_color_default));
		int electric_powerconsumption_appwidget_summary_font_size = resources.getDimensionPixelSize(R.dimen.electric_powerconsumption_appwidget_summary_font_size);

		// RemoteViews IuWFNg܂B
		RemoteViews remoteViews = new RemoteViews(this.getPackageName(), R.layout.electric_powerconsumption_appwidget);

		if (electric_powerconsumption_appwidget_background_enabled) {
			remoteViews.setImageViewBitmap(R.id.electric_powerconsumption_appwidget_background, this.getBackgroundBitmap());
		} else {
			remoteViews.setImageViewBitmap(R.id.electric_powerconsumption_appwidget_background, this.getTransparentBitmap());
		}

		// ڍ׏ SpannableStringBuilder ܂B
		SpannableStringBuilder electric_powerconsumption_appwidget_summary = new SpannableStringBuilder();

		electric_powerconsumption_appwidget_summary.append(resources.getString(R.string.electric_powerconsumption_appwidget_summary_loading_text));
		electric_powerconsumption_appwidget_summary.setSpan(new ForegroundColorSpan(electric_powerconsumption_appwidget_summary_font_color), 0, electric_powerconsumption_appwidget_summary.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
		electric_powerconsumption_appwidget_summary.setSpan(new AbsoluteSizeSpan(electric_powerconsumption_appwidget_summary_font_size, false), 0, electric_powerconsumption_appwidget_summary.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);

		// ɂ܂B
		if (electric_powerconsumption_appwidget_summary_font_bold) {
			electric_powerconsumption_appwidget_summary.setSpan(new StyleSpan(Typeface.BOLD), 0, electric_powerconsumption_appwidget_summary.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
		}

		// r[ɃeLXgƃCxgݒ肵܂B
		remoteViews.setTextViewText(R.id.electric_powerconsumption_appwidget_consumption_rate, EMPTY_STRING);
		remoteViews.setTextViewText(R.id.electric_powerconsumption_appwidget_summary, electric_powerconsumption_appwidget_summary);
		remoteViews.setTextViewText(R.id.electric_powerconsumption_appwidget_update_time, EMPTY_STRING);
		remoteViews.setOnClickPendingIntent(R.id.electric_powerconsumption_appwidget_consumption_rate, PendingIntent.getService(this, 0, new Intent(ACTION_APPWIDGET_CLICK), 0));
		remoteViews.setOnClickPendingIntent(R.id.electric_powerconsumption_appwidget_summary, PendingIntent.getService(this, 0, new Intent(ACTION_APPWIDGET_CLICK), 0));

		// AppWidgetManager ܂B
		AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(this);

		// BWFbgXV܂B
		appWidgetManager.updateAppWidget(new ComponentName(this, ElectricPowerConsumptionAppWidgetProvider.class), remoteViews);
	}

	/**
	 * ʂ\܂B
	 */
	private void refreshResult() {
		// LOG:
		Log.v(TAG, "void refreshResult()");

		// Resources, SharedPreferences IuWFNg擾܂B
		Resources resources = this.getResources();
		SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

		// ݒ擾܂B
		boolean electric_powerconsumption_appwidget_background_enabled = sharedPreferences.getBoolean(ELECTRIC_POWERCONSUMPTION_APPWIDGET_BACKGROUND_ENABLED, resources.getBoolean(R.bool.electric_powerconsumption_appwidget_background_enabled_default));
		boolean electric_powerconsumption_appwidget_consumption_rate_font_bold = sharedPreferences.getBoolean(ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_FONT_BOLD, resources.getBoolean(R.bool.electric_powerconsumption_appwidget_consumption_rate_font_bold_default));
		int electric_powerconsumption_appwidget_consumption_rate_font_color = sharedPreferences.getInt(ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_FONT_COLOR, resources.getColor(R.color.electric_powerconsumption_appwidget_consumption_rate_font_color_default));
		int electric_powerconsumption_appwidget_consumption_rate_font_size = resources.getDimensionPixelSize(R.dimen.electric_powerconsumption_appwidget_consumption_rate_font_size);
		boolean electric_powerconsumption_appwidget_consumption_rate_gradation_font_color_enabled = sharedPreferences.getBoolean(ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_GRADATION_FONT_COLOR_ENABLED, resources.getBoolean(R.bool.electric_powerconsumption_appwidget_consumption_rate_gradation_font_color_enabled_default));
		int electric_powerconsumption_appwidget_consumption_rate_zero_font_color = sharedPreferences.getInt(ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_ZERO_FONT_COLOR, resources.getColor(R.color.electric_powerconsumption_appwidget_consumption_rate_zero_font_color_default));
		int electric_powerconsumption_appwidget_consumption_rate_half_font_color = sharedPreferences.getInt(ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_HALF_FONT_COLOR, resources.getColor(R.color.electric_powerconsumption_appwidget_consumption_rate_half_font_color_default));
		int electric_powerconsumption_appwidget_consumption_rate_full_font_color = sharedPreferences.getInt(ELECTRIC_POWERCONSUMPTION_APPWIDGET_CONSUMPTION_RATE_FULL_FONT_COLOR, resources.getColor(R.color.electric_powerconsumption_appwidget_consumption_rate_full_font_color_default));
		boolean electric_powerconsumption_appwidget_summary_font_bold = sharedPreferences.getBoolean(ELECTRIC_POWERCONSUMPTION_APPWIDGET_SUMMARY_FONT_BOLD, resources.getBoolean(R.bool.electric_powerconsumption_appwidget_summary_font_bold_default));
		int electric_powerconsumption_appwidget_summary_font_color = sharedPreferences.getInt(ELECTRIC_POWERCONSUMPTION_APPWIDGET_SUMMARY_FONT_COLOR, resources.getColor(R.color.electric_powerconsumption_appwidget_summary_font_color_default));
		int electric_powerconsumption_appwidget_summary_font_size = resources.getDimensionPixelSize(R.dimen.electric_powerconsumption_appwidget_summary_font_size);

		// RemoteViews IuWFNg܂B
		RemoteViews remoteViews = new RemoteViews(this.getPackageName(), R.layout.electric_powerconsumption_appwidget);

		if (electric_powerconsumption_appwidget_background_enabled) {
			remoteViews.setImageViewBitmap(R.id.electric_powerconsumption_appwidget_background, this.getBackgroundBitmap());
		} else {
			remoteViews.setImageViewBitmap(R.id.electric_powerconsumption_appwidget_background, this.getTransparentBitmap());
		}

		int color;

		// p̐F肵܂B
		if (electric_powerconsumption_appwidget_consumption_rate_gradation_font_color_enabled) {
			float unit = this.today / this.peak;
			int from;
			int to;

			if (unit <= 0.5f) {
				from = electric_powerconsumption_appwidget_consumption_rate_zero_font_color;
				to = electric_powerconsumption_appwidget_consumption_rate_half_font_color;
			} else {
				from = electric_powerconsumption_appwidget_consumption_rate_half_font_color;
				to = electric_powerconsumption_appwidget_consumption_rate_full_font_color;
			}

			color = jp.gr.java_conf.k01.android.widgetpack.widget.ColorUtils.interp(from, to, unit);
		} else {
			color = electric_powerconsumption_appwidget_consumption_rate_font_color;
		}

		// p[Ze[W SpannableStringBuilder ܂B
		SpannableStringBuilder electric_powerconsumption_appwidget_consumption_rate = new SpannableStringBuilder();

		// p[Ze[W̕\\z܂B
		electric_powerconsumption_appwidget_consumption_rate.append(resources.getString(R.string.electric_powerconsumption_appwidget_consumption_rate_text_format, (this.today / this.peak) * 100f));
		electric_powerconsumption_appwidget_consumption_rate.setSpan(new ForegroundColorSpan(color), 0, electric_powerconsumption_appwidget_consumption_rate.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
		electric_powerconsumption_appwidget_consumption_rate.setSpan(new AbsoluteSizeSpan(electric_powerconsumption_appwidget_consumption_rate_font_size, false), 0, electric_powerconsumption_appwidget_consumption_rate.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);

		// ɐݒ肵܂B
		if (electric_powerconsumption_appwidget_consumption_rate_font_bold) {
			electric_powerconsumption_appwidget_consumption_rate.setSpan(new StyleSpan(Typeface.BOLD), 0, electric_powerconsumption_appwidget_consumption_rate.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
		}

		// ڍ׏ SpannableStringBuilder ܂B
		SpannableStringBuilder electric_powerconsumption_appwidget_summary = new SpannableStringBuilder();

		electric_powerconsumption_appwidget_summary.append(resources.getString(R.string.electric_powerconsumption_appwidget_summary_peak_text_format, this.peak));
		electric_powerconsumption_appwidget_summary.append(resources.getString(R.string.electric_powerconsumption_appwidget_summary_today_text_format, this.today));
		electric_powerconsumption_appwidget_summary.append(resources.getString(R.string.electric_powerconsumption_appwidget_summary_yesterday_text_format, this.yesterday));
		electric_powerconsumption_appwidget_summary.setSpan(new ForegroundColorSpan(electric_powerconsumption_appwidget_summary_font_color), 0, electric_powerconsumption_appwidget_summary.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
		electric_powerconsumption_appwidget_summary.setSpan(new AbsoluteSizeSpan(electric_powerconsumption_appwidget_summary_font_size, false), 0, electric_powerconsumption_appwidget_summary.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);

		// ɂ܂B
		if (electric_powerconsumption_appwidget_summary_font_bold) {
			electric_powerconsumption_appwidget_summary.setSpan(new StyleSpan(Typeface.BOLD), 0, electric_powerconsumption_appwidget_summary.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
		}

		// XV SpannableStringBuilder ܂B
		SpannableStringBuilder electric_powerconsumption_appwidget_update_time = new SpannableStringBuilder();

		electric_powerconsumption_appwidget_update_time.append(resources.getString(R.string.electric_powerconsumption_appwidget_summary_datetime_text_format, this.updateDate, this.updateTime));
		electric_powerconsumption_appwidget_update_time.setSpan(new ForegroundColorSpan(electric_powerconsumption_appwidget_summary_font_color), 0, electric_powerconsumption_appwidget_update_time.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
		electric_powerconsumption_appwidget_update_time.setSpan(new AbsoluteSizeSpan(electric_powerconsumption_appwidget_summary_font_size, false), 0, electric_powerconsumption_appwidget_update_time.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);

		// ɐݒ肵܂B
		if (electric_powerconsumption_appwidget_summary_font_bold) {
			electric_powerconsumption_appwidget_update_time.setSpan(new StyleSpan(Typeface.BOLD), 0, electric_powerconsumption_appwidget_update_time.length(), Spanned.SPAN_EXCLUSIVE_INCLUSIVE);
		}

		// r[ɃeLXgƃCxgݒ肵܂B
		remoteViews.setTextViewText(R.id.electric_powerconsumption_appwidget_consumption_rate, electric_powerconsumption_appwidget_consumption_rate);
		remoteViews.setTextViewText(R.id.electric_powerconsumption_appwidget_summary, electric_powerconsumption_appwidget_summary);
		remoteViews.setTextViewText(R.id.electric_powerconsumption_appwidget_update_time, electric_powerconsumption_appwidget_update_time);
		remoteViews.setOnClickPendingIntent(R.id.electric_powerconsumption_appwidget_consumption_rate, PendingIntent.getService(this, 0, new Intent(ACTION_APPWIDGET_CLICK), 0));
		remoteViews.setOnClickPendingIntent(R.id.electric_powerconsumption_appwidget_summary, PendingIntent.getService(this, 0, new Intent(ACTION_APPWIDGET_CLICK), 0));

		// AppWidgetManager ܂B
		AppWidgetManager appWidgetManager = AppWidgetManager.getInstance(this);

		// BWFbgXV܂B
		appWidgetManager.updateAppWidget(new ComponentName(this, ElectricPowerConsumptionAppWidgetProvider.class), remoteViews);
	}

	/**
	 * AlarmManager ɃXPW[o^܂B
	 */
	private void registerAlarmManager() {
		// LOG:
		Log.v(TAG, "void registerAlarmManager()");

		SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

		// ݒl擾܂B
		int electric_powerconsumption_appwidget_network_interval_minute = sharedPreferences.getInt(ELECTRIC_POWERCONSUMPTION_APPWIDGET_NETWORK_INTERVAL, this.getResources().getInteger(R.integer.electric_powerconsumption_appwidget_network_interval_minute_default));

		// AlarmManager Ƀs[g^C}[ݒ肵܂B
		AlarmManager alarmManager = (AlarmManager) this.getSystemService(Context.ALARM_SERVICE);

		Calendar calendar = Calendar.getInstance();

		calendar.setTimeInMillis(System.currentTimeMillis());
		calendar.roll(Calendar.HOUR, true);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);

		// ~bɕϊ܂B
		long timeInMillis = electric_powerconsumption_appwidget_network_interval_minute * 60 * 1000;

		// LOG:
		Log.v(TAG, String.format("Interval Millis: %d", timeInMillis));

		Intent intent = new Intent(ACTION_APPWIDGET_INVALIDATE, null, this, ElectricPowerConsumptionAppWidgetService.class);

		// Ώۂ Intent ܂B
		alarmManager.setRepeating(AlarmManager.RTC, calendar.getTimeInMillis(), timeInMillis, PendingIntent.getService(this, 0, intent, 0));
	}

	/**
	 * NotificationManager ɒʒmbZ[W̕\w܂B
	 * 
	 * @param tickerText
	 *            eBbJ[̈ɕ\eLXgB
	 * @param title
	 *            ʒm̈ɕ\^CgB
	 * @param message
	 *            ʒm̈ɕ\郁bZ[WB
	 */
	private void showNotification(String tickerText, String title, String message) {
		// NotificationManager 擾܂B
		NotificationManager notificationManager = (NotificationManager) this.getSystemService(Context.NOTIFICATION_SERVICE);

		// Notification ܂B
		Notification notification = new Notification(R.drawable.icon, tickerText, System.currentTimeMillis());
		PendingIntent intent = PendingIntent.getService(this, 0, new Intent(ACTION_NOTIFICATION_CLICK, null, this, ElectricPowerConsumptionAppWidgetService.class), 0);

		notification.setLatestEventInfo(this, title, message, intent);
		notification.flags |= Notification.FLAG_AUTO_CANCEL;

		// Notification ʒm܂B
		notificationManager.notify(R.string.electric_powerconsumption_appwidget_name, notification);
	}

	/**
	 * AlarmManager ̃XPW[LZ܂B
	 */
	private void unregisterAlarmManager() {
		// LOG:
		Log.v(TAG, "void unregisterAlarmManager()");

		// AlarmManager ̐ݒLZ܂B
		AlarmManager alarmManager = (AlarmManager) this.getSystemService(Context.ALARM_SERVICE);

		Intent intent = new Intent(ACTION_APPWIDGET_INVALIDATE, null, this, ElectricPowerConsumptionAppWidgetService.class);

		alarmManager.cancel(PendingIntent.getService(this, 0, intent, 0));
	}

	/**
	 * f[^XV܂B
	 */
	public void update() {
		// LOG:
		Log.v(TAG, "update()");

		if (this.isLoading) {
			return;
		}

		// Resources, SharedPreferences IuWFNg擾܂B
		Resources resources = this.getResources();
		SharedPreferences sharedPreferences = PreferenceManager.getDefaultSharedPreferences(this);

		// ݒl擾܂B
		boolean electric_powerconsumption_appwidget_notification_timeout_enabled = resources.getBoolean(R.bool.electric_powerconsumption_appwidget_notification_timeout_enabled_default);
		int electric_powerconsumption_appwidget_network_suspend_millis = resources.getInteger(R.integer.electric_powerconsumption_appwidget_network_suspend_millis_default);

		try {
			// tOAG[̗L܂B
			this.isLoading = true;
			this.exception = null;

			String tickerText = null;
			String title = null;
			String message = null;

			// TXyhԓmF܂B
			if ((System.currentTimeMillis() - this.lastModificationTime) < electric_powerconsumption_appwidget_network_suspend_millis) {
				tickerText = this.getString(R.string.electric_powerconsumption_appwidget_notification_update_suspend_ticker_text);
				title = this.getString(R.string.electric_powerconsumption_appwidget_notification_update_suspend_title);
				message = this.getString(R.string.electric_powerconsumption_appwidget_notification_update_suspend);
			} else {
				// XVObZ[W\܂B
				this.showNotification(this.getString(R.string.electric_powerconsumption_appwidget_notification_update_before_ticker_text), this.getString(R.string.electric_powerconsumption_appwidget_notification_update_before_title), this.getString(R.string.electric_powerconsumption_appwidget_notification_update_before));

				// HttpClient ܂B
				HttpClient httpClient = new DefaultHttpClient();

				// HttpGet ܂B
				HttpGet httpGet = new HttpGet(sharedPreferences.getString(ELECTRIC_POWERCONSUMPTION_APPWIDGET_NETWORK_URL, this.getString(R.string.electric_powerconsumption_appwidget_network_url_default)));

				// HttpResponse 擾܂B
				HttpResponse httpResponse = httpClient.execute(httpGet);

				// X|XR[hmF܂B
				if (httpResponse.getStatusLine().getStatusCode() != HttpStatus.SC_OK) {
					throw new IOException(String.format("Invalidate status code %d.", httpResponse.getStatusLine().getStatusCode()));
				}

				// f[^擾܂B
				String s = EntityUtils.toString(httpResponse.getEntity(), DATA_ENCODING);

				BufferedReader reader = new BufferedReader(new StringReader(s));
				String line = null;
				String[] cells = null;
				int state = 0;

				while ((line = reader.readLine()) != null) {
					// LOG:
					Log.v(TAG, String.format("LINE: %s", line));

					if (line.length() <= 0) {
						continue;
					}

					switch (state) {
					case 0:
						if (line.startsWith(CSV_STARTWITH_PEAK_POWER)) {
							state = 1;
						} else {
							cells = line.split(CSV_SEPARATOR_SPACE);

							this.updateDate = cells[0];
							this.updateTime = cells[1];
						}
						break;
					case 1:
						if (line.startsWith(CSV_STARTWITH_EXPECTED_POWER)) {
							state = 2;
						} else {
							cells = line.split(CSV_SEPARATOR);

							this.peak = Float.parseFloat(cells[0]);
						}
						break;
					case 2:
						if (line.startsWith(CSV_STARTWITH_DATE)) {
							state = 3;
						}
						break;
					case 3:
						cells = line.split(CSV_SEPARATOR);

						float today = Float.parseFloat(cells[2]);
						float yesterday = Float.parseFloat(cells[3]);

						if (today != 0) {
							this.today = today;
							this.yesterday = yesterday;
						}

						break;
					}
				}

				// LOG:
				Log.v(TAG, String.format("XV: %s %s", this.updateDate, this.updateTime));
				Log.v(TAG, String.format("s[N: %f", this.peak));
				Log.v(TAG, String.format("{: %f, : %f", this.today, this.yesterday));

				tickerText = this.getString(R.string.electric_powerconsumption_appwidget_notification_update_success_ticker_text);
				title = this.getString(R.string.electric_powerconsumption_appwidget_notification_update_success_title);
				message = this.getString(R.string.electric_powerconsumption_appwidget_notification_update_success, this.updateDate, this.updateTime);

				// ŏIXVێ܂B
				this.lastModificationTime = System.currentTimeMillis();
			}

			this.isReady = true;
			this.isLoading = false;

			// Notification ʒm܂B
			this.showNotification(tickerText, title, message);

			if (electric_powerconsumption_appwidget_notification_timeout_enabled) {
				this.autoCancelNotification();
			}
		} catch (Exception ex) {
			// LOG:
			Log.e(TAG, String.format("Exception message is %s (%s).", ex.getMessage(), ex.getClass().getName()));

			this.isReady = true;
			this.isLoading = false;
			this.exception = ex;

			// Notification ʒm܂B
			if (ex instanceof IOException) {
				this.showNotification(this.getString(R.string.electric_powerconsumption_appwidget_notification_update_error_ticker_text), this.getString(R.string.electric_powerconsumption_appwidget_notification_update_error_title), this.getString(R.string.electric_powerconsumption_appwidget_notification_update_error_io));
			} else {
				this.showNotification(this.getString(R.string.electric_powerconsumption_appwidget_notification_update_error_ticker_text), this.getString(R.string.electric_powerconsumption_appwidget_notification_update_error_title), this.getString(R.string.electric_powerconsumption_appwidget_notification_update_error_unknown));
			}

			if (electric_powerconsumption_appwidget_notification_timeout_enabled) {
				this.autoCancelNotification();
			}
		}
	}

}
